@@ -1,4 +1,24 @@
 #!/usr/local/bin/python2.7
+#
+#
+# Licensed to the Apache Software Foundation (ASF) under one
+# or more contributor license agreements.  See the NOTICE file
+# distributed with this work for additional information
+# regarding copyright ownership.  The ASF licenses this file
+# to you under the Apache License, Version 2.0 (the
+# "License"); you may not use this file except in compliance
+# with the License.  You may obtain a copy of the License at
+#
+#   http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing,
+# software distributed under the License is distributed on an
+# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
+# KIND, either express or implied.  See the License for the
+# specific language governing permissions and limitations
+# under the License.
+#
+#
 """This is a pre-commit hook that checks whether the contents of PO files
 committed to the repository are encoded in UTF-8.
 """
@@ -76,7 +96,7 @@ class ChangeReceiver(delta.Editor):
         stream = core.Stream(fs.file_contents(self.txn_root, path, subpool))
         reader = codecs.getreader('UTF-8')(stream, 'strict')
         writer = codecs.getwriter('UTF-8')(checker, 'strict')
-        while 1:
+        while True:
           data = reader.read(core.SVN_STREAM_CHUNK_SIZE)
           if not data:
             break
